/**
 * @file
 * VuoList_VuoWave C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoWave_H
#define VuoList_VuoWave_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoWave VuoList_VuoWave
 * A list of @ref VuoWave elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoWave_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoWave elements.
 */
typedef const struct VuoList_VuoWave_struct { void *l; } * VuoList_VuoWave;
/// @}
#endif

/**
 * Creates a new list of @ref VuoWave elements.
 */
VuoList_VuoWave VuoListCreate_VuoWave(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoWave VuoListCopy_VuoWave(const VuoList_VuoWave list);

/**
 * Returns the @ref VuoWave at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoWave VuoListGetValue_VuoWave(const VuoList_VuoWave list, const unsigned long index);

/**
 * Changes the @ref VuoWave at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoWave(const VuoList_VuoWave list, const VuoWave value, const unsigned long index);

/**
 * Inserts the `VuoWave` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoWave(const VuoList_VuoWave list, const VuoWave value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoWave(VuoList_VuoWave list, const VuoWave value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoWave(VuoList_VuoWave list, const VuoWave value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoWave(VuoList_VuoWave list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoWave(VuoList_VuoWave list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoWave(VuoList_VuoWave list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoWave(VuoList_VuoWave list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoWave(VuoList_VuoWave list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoWave(VuoList_VuoWave list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoWave(VuoList_VuoWave list);

/**
 * Removes the `VuoWave` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoWave(VuoList_VuoWave list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoWave(const VuoList_VuoWave list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoWave VuoList_VuoWave_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoWave_getJson(const VuoList_VuoWave value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoWave_getSummary(const VuoList_VuoWave value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoWave VuoList_VuoWave_makeFromString(const char *str);
char * VuoList_VuoWave_getString(const VuoList_VuoWave value);
/// @}

/**
 * @}
 */

#endif
