/**
 * @file
 * VuoList_VuoWindowProperty C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoWindowProperty_H
#define VuoList_VuoWindowProperty_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoWindowProperty VuoList_VuoWindowProperty
 * A list of @ref VuoWindowProperty elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoWindowProperty_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoWindowProperty elements.
 */
typedef const struct VuoList_VuoWindowProperty_struct { void *l; } * VuoList_VuoWindowProperty;
/// @}
#endif

/**
 * Creates a new list of @ref VuoWindowProperty elements.
 */
VuoList_VuoWindowProperty VuoListCreate_VuoWindowProperty(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoWindowProperty VuoListCopy_VuoWindowProperty(const VuoList_VuoWindowProperty list);

/**
 * Returns the @ref VuoWindowProperty at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoWindowProperty VuoListGetValue_VuoWindowProperty(const VuoList_VuoWindowProperty list, const unsigned long index);

/**
 * Changes the @ref VuoWindowProperty at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoWindowProperty(const VuoList_VuoWindowProperty list, const VuoWindowProperty value, const unsigned long index);

/**
 * Inserts the `VuoWindowProperty` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoWindowProperty(const VuoList_VuoWindowProperty list, const VuoWindowProperty value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoWindowProperty(VuoList_VuoWindowProperty list, const VuoWindowProperty value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoWindowProperty(VuoList_VuoWindowProperty list, const VuoWindowProperty value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoWindowProperty(VuoList_VuoWindowProperty list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoWindowProperty(VuoList_VuoWindowProperty list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoWindowProperty(VuoList_VuoWindowProperty list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoWindowProperty(VuoList_VuoWindowProperty list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoWindowProperty(VuoList_VuoWindowProperty list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoWindowProperty(VuoList_VuoWindowProperty list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoWindowProperty(VuoList_VuoWindowProperty list);

/**
 * Removes the `VuoWindowProperty` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoWindowProperty(VuoList_VuoWindowProperty list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoWindowProperty(const VuoList_VuoWindowProperty list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoWindowProperty VuoList_VuoWindowProperty_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoWindowProperty_getJson(const VuoList_VuoWindowProperty value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoWindowProperty_getSummary(const VuoList_VuoWindowProperty value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoWindowProperty VuoList_VuoWindowProperty_makeFromString(const char *str);
char * VuoList_VuoWindowProperty_getString(const VuoList_VuoWindowProperty value);
/// @}

/**
 * @}
 */

#endif
